/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/
package de.frankmuenster.jameica.orverw.db.server;

import java.rmi.RemoteException;

import de.frankmuenster.jameica.orverw.db.rmi.Version;
import de.frankmuenster.jameica.orverw.util.I18N;
import de.willuhn.datasource.db.AbstractDBObject;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Implementierung f&uuml;r einen Datensatz in der Versionstabelle.
 *
 * <pre>
 * CREATE TABLE version (
 *  id NUMERIC default UNIQUEKEY('version'),
 *  name varchar(255) NOT NULL,
 *  version int(5) NOT NULL,
 *  UNIQUE (id),
 *  PRIMARY KEY (id)
 * );
 * </pre>
 *
 * Getters und Setters f&uuml;r den Primary Key (id) sind nicht n&ouml;tig. Jede
 * Methode muss zwingend eine RemoteException werfen. <br>
 *
 * @author Frank Mnster
 *
 */
public class VersionImpl extends AbstractDBObject implements Version {

    /**
     * The version table column name
     */
    private static final String VERSION = "version"; //$NON-NLS-1$

    /**
     * serialVersionUID
     */
    private static final long serialVersionUID = -7949815027870224311L;

    /**
     * Standart Konstruktor
     *
     * @throws RemoteException
     */
    public VersionImpl() throws RemoteException {
	super();
    }

    /**
     * @see de.frankmuenster.jameica.orverw.db.rmi.Version#getName()
     */
    @Override
    public String getName() throws RemoteException {
	return (String) getAttribute("name"); //$NON-NLS-1$
    }

    /**
     * @see de.willuhn.datasource.db.AbstractDBObject#getPrimaryAttribute()
     */
    @Override
    public String getPrimaryAttribute() throws RemoteException {
	return "name"; //$NON-NLS-1$
    }

    /**
     * @see de.frankmuenster.jameica.orverw.db.rmi.Version#getVersion()
     */
    @Override
    public int getVersion() throws RemoteException {
	// Wir fangen bei 0 an mit dem Zaehlen
	Integer version;
	try {
	    version = (Integer) getAttribute(VersionImpl.VERSION);
	    if (version == null) {
		version = 0; // NOPMD
	    }
	} catch (final RemoteException e) {
	    version = 0;
	}
	return version.intValue();
    }

    /**
     * @see de.willuhn.datasource.db.AbstractDBObject#insert()
     */
    @Override
    public void insert() throws RemoteException, ApplicationException {
	setVersion(getVersion()); // speichert automatisch die Startnummer
	super.insert();
    }

    /**
     * @see de.frankmuenster.jameica.orverw.db.rmi.Version#setName(java.lang.String)
     */
    @Override
    public void setName(final String name) throws RemoteException {
	setAttribute("name", name); //$NON-NLS-1$
    }

    /**
     * @see de.frankmuenster.jameica.orverw.db.rmi.Version#setVersion(int)
     */
    @Override
    public void setVersion(final int newVersion) throws RemoteException {
	if (newVersion < 0) {
	    throw new RemoteException(I18N.tr("VersionImpl.versionBelowZero")); //$NON-NLS-1$
	}
	setAttribute(VersionImpl.VERSION, Integer.valueOf(newVersion));
    }

    /**
     * @see de.willuhn.datasource.db.AbstractDBObject#getTableName()
     */
    @Override
    protected String getTableName() {
	return VersionImpl.VERSION;
    }

    /**
     * @see de.willuhn.datasource.db.AbstractDBObject#insertCheck()
     */
    @Override
    protected void insertCheck() throws ApplicationException {

	try {
	    if (getName() == null || getName().length() == 0) {
		throw new ApplicationException(I18N.tr("VersionImpl.versionHasNoName")); //$NON-NLS-1$
	    }
	} catch (final RemoteException re) {
	    Logger.error(I18N.tr("VersionImpl.errorCheckingVersion"), re); //$NON-NLS-1$
	    throw new ApplicationException(I18N.tr("VersionImpl.errorCheckingVersion"), re); //$NON-NLS-1$
	}
	super.insertCheck();
    }

    /**
     * @see de.willuhn.datasource.db.AbstractDBObject#updateCheck()
     */
    @Override
    protected void updateCheck() throws ApplicationException {
	insertCheck();
    }

}
