/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.orverw.gui.input;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.ColorDialog;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.FontDialog;

import de.frankmuenster.jameica.orverw.util.I18N;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.input.AbstractInput;
import de.willuhn.jameica.gui.util.SWTUtil;
import de.willuhn.util.ApplicationException;

/**
 * An input element to gather text style information to format a text. The
 * following style information is treated: <br />
 * <ul>
 * <li>text font</li>
 * <li>text colour</li>
 * <li>background colour</li>
 * <li>text alignment</li>
 * </ul>
 *
 * @author Frank Mnster
 *
 */
@SuppressWarnings({ "PMD.GodClass" })
public class TextStyleInput extends AbstractInput implements SelectionListener {

    /**
     * Show foreground colour button
     */
    public static final int FOREGROUND = 1;
    /**
     * Show background colour button
     */
    public static final int BACKGROUND = 2;
    /**
     * Show alignment buttons
     */
    public static final int ALIGNMENT = 4;

    /**
     * Der R&uuml<gabewewrt
     */
    private TextStyle value;

    /**
     * Der SWT Kontainer f&uuml;r das Input
     */
    private transient Composite container;

    /**
     * Das Beispieltext Label
     */
    private transient CLabel example;

    /**
     * Die Breite des Beispieltextfeldes
     */
    private transient int exampleWidth;

    /**
     * Der Button zum Anzeigen des Schriftauswahldialoges
     */
    private transient Button font;

    /**
     * Der Button zur Anzeigen des Schriftfarbendialoges
     */
    private transient Button textColor;

    /**
     * Der Button zur Anzeigen des Hintergrunffarbendialoges
     */
    private transient Button backgroundColor;

    /**
     * Der Text-Links-Ausrichten-Button
     */
    private transient Button left;

    /**
     * Der Text-Zentrieren-Button
     */
    private transient Button center;

    /**
     * Der Text-Rechts-Ausrichten-Button
     */
    private transient Button right;

    /**
     * Sichtbarkeit des Textfarben-Button
     */
    private transient boolean showFgButton;

    /**
     * Sichtbarkeit des Hintergrundfarben-Button
     */
    private transient boolean showBgButton;

    /**
     * Sichtbarkeit des Textausrichtungs-Buttons
     */
    private transient boolean showAlignButtons;

    /**
     * Haben wir den Fokus?
     */
    private transient boolean hasFocus;

    /**
     * Sind wir aktiviert?
     */
    private boolean enabled;

    /**
     * Erzeugt ein TextStyleInput Part mit allen Buttons sichtbar.
     *
     * @param style
     *            das TextStyle Object, dass als Grundlage dient.
     * @throws ApplicationException
     */
    public TextStyleInput(final TextStyle style) throws ApplicationException {
	this(style, TextStyleInput.FOREGROUND | TextStyleInput.BACKGROUND | TextStyleInput.ALIGNMENT,
		new Point(200, 0));
    }

    /**
     * Erzeugt ein TextStyleInput Part mit den angegebenen Buttons.
     *
     * @param style
     *            das TextStyle Object, dass als Grundlage dient.
     * @param showOption
     *            Buttons die sichtbar sind. M&ouml;gliche Werte sind
     *            TextStyleInput.FOREGROUND, TextStyleInput.BACKGROUND,
     *            TextStyleInput.ALIGNMENT.
     * @throws ApplicationException
     */
    public TextStyleInput(final TextStyle style, final int showOption) throws ApplicationException {

	this(style, showOption, new Point(200, 0));

    }

    /**
     * Erzeugt ein TextStyleInput Part mit den angegebenen Buttons.
     *
     * @param style
     *            das TextStyle Object, dass als Grundlage dient.
     * @param showOption
     *            Buttons die sichtbar sind. M&ouml;gliche Werte sind
     *            TextStyleInput.FOREGROUND, TextStyleInput.BACKGROUND,
     *            TextStyleInput.ALIGNMENT.
     * @param exampleWidth
     *            Breite des Beispielfeldes
     * @throws ApplicationException
     */
    public TextStyleInput(final TextStyle style, final int showOption, final Point exampleWidth)
	    throws ApplicationException {

	super();

	if (style == null) {
	    new ApplicationException(I18N.tr("TextStyleInput.styleIsNull")); //$NON-NLS-1$
	}
	this.value = style;
	this.showFgButton = (showOption & TextStyleInput.FOREGROUND) == TextStyleInput.FOREGROUND;
	this.showBgButton = (showOption & TextStyleInput.BACKGROUND) == TextStyleInput.BACKGROUND;
	this.showAlignButtons = (showOption & TextStyleInput.ALIGNMENT) == TextStyleInput.ALIGNMENT;
	this.exampleWidth = exampleWidth.x;

    }

    /**
     * Erzeugt ein TextStyleInput Part mit allen Buttons sichtbar.
     *
     * @param style
     *            das TextStyle Object, dass als Grundlage dient.
     * @param exampleWidth
     *            Breite des Beispielfeldes
     * @throws ApplicationException
     */
    public TextStyleInput(final TextStyle style, final Point exampleWidth) throws ApplicationException {
	this(style, TextStyleInput.FOREGROUND | TextStyleInput.BACKGROUND | TextStyleInput.ALIGNMENT, exampleWidth);
    }

    /**
     * @see de.willuhn.jameica.gui.input.Input#disable()
     */
    @Override
    public void disable() {
	setEnabled(false);
    }

    /**
     * @see de.willuhn.jameica.gui.input.Input#enable()
     */
    @Override
    public void enable() {
	setEnabled(true);
    }

    /**
     * @see de.willuhn.jameica.gui.input.Input#focus()
     */
    @Override
    public void focus() {
	this.hasFocus = true;
	if (this.container != null && !this.container.isDisposed()) {
	    this.container.setFocus();
	}
    }

    /**
     * @see de.willuhn.jameica.gui.input.Input#getControl()
     */
    @Override
    public Control getControl() {

	if (this.container != null) {
	    return this.container; // NOPMD
	}

	this.container = new Composite(getParent(), SWT.NONE);
	initContainer();
	getExample();
	getButtons();

	if (this.hasFocus) {
	    this.container.setFocus();
	}
	return this.container;
    }

    /**
     * @see de.willuhn.jameica.gui.input.Input#getValue()
     */
    @Override
    public Object getValue() {
	return this.value;

    }

    /**
     * @see de.willuhn.jameica.gui.input.Input#isEnabled()
     */
    @Override
    public boolean isEnabled() {
	Boolean isEnabled;
	if (this.container == null || this.container.isDisposed()) {
	    isEnabled = this.enabled;
	} else {
	    isEnabled = this.container.getEnabled();
	}
	return isEnabled;
    }

    /**
     * @see de.willuhn.jameica.gui.input.Input#setEnabled(boolean)
     */
    @Override
    public void setEnabled(final boolean enabled) {
	this.enabled = enabled;
	if (this.container != null && !this.container.isDisposed()) {
	    this.container.setEnabled(enabled);
	}
    }

    /**
     * @see de.willuhn.jameica.gui.input.Input#setValue(java.lang.Object)
     */
    @Override
    public void setValue(final Object value) {
	if (!(value instanceof TextStyle)) {
	    return;
	}

	this.value = (TextStyle) value;
	if (!this.showAlignButtons) {
	    this.value.setAlignment(SWT.LEFT);
	}

	if (this.container != null && !this.container.isDisposed()) {
	    redraw();
	}

    }

    /**
     * @see org.eclipse.swt.events.SelectionListener#widgetDefaultSelected(org.eclipse
     *      .swt.events.SelectionEvent)
     */
    @Override
    public void widgetDefaultSelected(final SelectionEvent arg0) {
	// Nothing to do
    }

    /**
     * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse.swt
     *      .events.SelectionEvent)
     */
    @Override
    public void widgetSelected(final SelectionEvent event) {

	final Object source = event.getSource();
	if (source instanceof Button) {
	    handleButtonClicked((Button) source);
	    redraw();
	}

    }

    private void getButtons() {
	this.font = new Button(this.container, SWT.PUSH);
	this.font.setText(I18N.tr("TextStyleInput.font")); //$NON-NLS-1$
	this.font.addSelectionListener(this);

	if (this.showFgButton) {
	    this.textColor = new Button(this.container, SWT.PUSH);
	    this.textColor.setImage(SWTUtil.getImage("colors-icon.png")); //$NON-NLS-1$
	    this.textColor.addSelectionListener(this);
	    this.textColor.setToolTipText(I18N.tr("TextStyleInput.textColour")); //$NON-NLS-1$
	}

	if (this.showAlignButtons) {
	    this.left = new Button(this.container, SWT.TOGGLE | SWT.CENTER);
	    this.left.setImage(SWTUtil.getImage("align_left.png")); //$NON-NLS-1$
	    this.left.setSelection(true);
	    this.left.addSelectionListener(this);

	    this.center = new Button(this.container, SWT.TOGGLE | SWT.CENTER);
	    this.center.setImage(SWTUtil.getImage("align_center.png")); //$NON-NLS-1$
	    this.center.addSelectionListener(this);

	    this.right = new Button(this.container, SWT.TOGGLE | SWT.CENTER);
	    this.right.setImage(SWTUtil.getImage("align_right.png")); //$NON-NLS-1$
	    this.right.addSelectionListener(this);
	}

	if (this.showBgButton) {
	    this.backgroundColor = new Button(this.container, SWT.PUSH);
	    this.backgroundColor.setImage(SWTUtil.getImage("colors-icon.png")); //$NON-NLS-1$
	    this.backgroundColor.addSelectionListener(this);
	    this.backgroundColor.setToolTipText(I18N.tr("TextStyleInput.bgColour")); //$NON-NLS-1$

	}
    }

    private void getExample() {
	this.example = new CLabel(this.container, SWT.NONE);
	final GridData gridData = new GridData();
	gridData.widthHint = this.exampleWidth;
	gridData.minimumWidth = 40;
	this.example.setLayoutData(gridData);
	this.example.setTopMargin(4);
	this.example.setBottomMargin(4);
	this.example.setText(I18N.tr("TextStyleInput.exampleText")); //$NON-NLS-1$
	this.example.setFont(this.value.getFont());
	this.example.setForeground(this.value.getTextColor());
	this.example.setBackground(this.value.getBackgroundColor());
	this.example.setAlignment(this.value.getAlignment());
    }

    private void handleAlignment(final Button button) {

	if (button.equals(this.left)) {
	    this.value.setAlignment(SWT.LEFT);
	} else if (button.equals(this.center)) {
	    this.value.setAlignment(SWT.CENTER);
	} else if (button.equals(this.right)) {
	    this.value.setAlignment(SWT.RIGHT);
	}

    }

    private void handleButtonClicked(final Button button) {

	handleFontColourButtons(button);
	handleAlignment(button);
	redraw();

    }

    private void handleFontColourButtons(final Button button) {

	if (button.equals(this.font)) {
	    final FontDialog fontDialog = new FontDialog(GUI.getShell());
	    fontDialog.setEffectsVisible(false);
	    fontDialog.setFontList(this.value.getFont().getFontData());
	    final FontData fontData = fontDialog.open();
	    if (fontData != null) {
		final Font fontNew = new Font(GUI.getDisplay(), fontData);
		this.value.setFont(fontNew);
	    }
	} else if (button.equals(this.textColor)) {
	    final ColorDialog colorDialog = new ColorDialog(GUI.getShell());
	    colorDialog.setRGB(this.value.getTextColor().getRGB());
	    final RGB colorRGB = colorDialog.open();
	    if (colorRGB != null) {
		final Color color = new Color(GUI.getDisplay(), colorRGB);
		this.value.setTextColor(color);
	    }
	} else if (button.equals(this.backgroundColor)) {
	    final ColorDialog colorDialog = new ColorDialog(GUI.getShell());
	    colorDialog.setRGB(this.value.getBackgroundColor().getRGB());
	    final RGB colorRGB = colorDialog.open();
	    if (colorRGB != null) {
		final Color color = new Color(GUI.getDisplay(), colorRGB);
		this.value.setBackgroundColor(color);
	    }
	}
    }

    private void initContainer() {
	int colums = 7;
	if (!this.showFgButton) {
	    colums--;
	}
	if (!this.showAlignButtons) {
	    colums -= 3;
	}
	if (!this.showBgButton) {
	    colums--;
	}
	this.container.setLayout(new GridLayout(colums, false));

    }

    private void redraw() {

	// Font
	this.example.setFont(this.value.getFont());
	this.example.setForeground(this.value.getTextColor());
	this.example.setBackground(this.value.getBackgroundColor());
	this.example.setAlignment(this.value.getAlignment());
	if (this.left != null && this.center != null && this.right != null) {
	    switch (this.value.getAlignment()) {
	    case SWT.LEFT:
		this.left.setSelection(true);
		this.center.setSelection(false);
		this.right.setSelection(false);
		break;
	    case SWT.CENTER:
		this.left.setSelection(false);
		this.center.setSelection(true);
		this.right.setSelection(false);
		break;
	    case SWT.RIGHT:
		this.left.setSelection(false);
		this.center.setSelection(false);
		this.right.setSelection(true);
		break;
	    default:
		break;
	    }
	}

	// Layout anpassen
	GUI.getCurrentView().getParent().layout();

    }
}
