/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.orverw.print;

import java.rmi.RemoteException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import org.apache.commons.lang.StringUtils;
import org.eclipse.nebula.paperclips.core.ImagePrint;
import org.eclipse.nebula.paperclips.core.LayerPrint;
import org.eclipse.nebula.paperclips.core.Print;
import org.eclipse.nebula.paperclips.core.grid.DefaultGridLook;
import org.eclipse.nebula.paperclips.core.grid.GridPrint;
import org.eclipse.nebula.paperclips.core.page.PageNumber;
import org.eclipse.nebula.paperclips.core.page.PageNumberPrint;
import org.eclipse.nebula.paperclips.core.page.PagePrint;
import org.eclipse.nebula.paperclips.core.text.TextPrint;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.Point;

import de.frankmuenster.jameica.orverw.db.rmi.Ordner;
import de.frankmuenster.jameica.orverw.db.rmi.Register;
import de.frankmuenster.jameica.orverw.util.I18N;
import de.frankmuenster.jameica.orverw.util.TextUtils;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Implementiert den Ausdruck des Registers.
 *
 * @author Frank Mnster
 * @since 1.0
 */
@SuppressWarnings({ "PMD.ExcessiveImports", "PMD.BeanMembersShouldSerialize" })
public class PrintRegister extends AbstractPrint {

    /**
     * Maximal width of description field in px
     */
    public static final int DESCRIPTION_WIDTH = 340;

    /**
     * Der Registerinhalt
     */
    private final DBIterator<Register> index;

    /**
     * Der Ordner Titel
     */
    private final String title;

    /**
     * Die Ordner ID
     */
    private final String ordnerID;

    /**
     * Das letzte Speicherdatum
     */
    private final Date lastSaveDate;

    /**
     * Der Titeltext Schrifttyp
     */
    private final Font titleFont;

    /**
     * der Indextext Schrifttyp
     */
    private final Font indexFont;

    /**
     * Der Beschreibungstext Schrifttyp
     */
    private final Font contentFont;

    /**
     * Die Hintergrundfarbe des Indexfeldes
     */
    private final Color indexBgColor;

    /**
     * Die Hintergrundfarbe des Beschreibungsfeldes
     */
    private final Color contentBgColor;

    /**
     * Der Konstruktor
     *
     * @param ordner
     *            der Ordner dessen Register gedruckt werden soll.
     * @throws ApplicationException
     */
    public PrintRegister(final Ordner ordner) throws ApplicationException {
	super();
	try {
	    this.index = ordner.getInhaltsverzeichniss();
	    this.title = ordner.getTitel();
	    this.ordnerID = ordner.getID();
	    this.lastSaveDate = ordner.getDate();
	    this.titleFont = ordner.getTitelFont();
	    this.indexFont = ordner.getRegisterIndexFont();
	    this.contentFont = ordner.getRegisterContentFont();
	    this.indexBgColor = ordner.getRegisterIndexBackground();
	    this.contentBgColor = ordner.getRegisterContentBackground();
	} catch (final RemoteException e) {
	    throw new ApplicationException(I18N.tr("PrintRegister.errorCreatingPrintObj"), e); //$NON-NLS-1$
	}
    }

    /**
     * @see de.frankmuenster.jameica.orverw.print.AbstractPrint#getTitle()
     */
    @Override
    public String getTitle() {

	return TextUtils.format(this.title, 10);
    }

    /**
     * @see AbstractPrint#customise(PagePrint)
     */
    @Override
    protected void customise(final PagePrint page) throws ApplicationException {

	// Neuer Header mit ID
	page.setHeader((final PageNumber pageNumber) -> {

	    final GridPrint header = new GridPrint("d:g,r:d"); //$NON-NLS-1$
	    final TextPrint title1 = new TextPrint(StringUtils.EMPTY, AbstractPrint.FONT_TINY, SWT.LEFT);
	    final TextPrint title2 = new TextPrint("ID " + this.ordnerID, AbstractPrint.FONT_TINY, //$NON-NLS-1$
		    SWT.RIGHT);
	    final TextPrint title3 = new TextPrint(this.title, this.titleFont.getFontData()[0]);
	    header.add(SWT.RIGHT, title2, 2);
	    header.add(SWT.LEFT, title3, 2);
	    header.add(SWT.RIGHT, title1, 2);
	    header.add(SWT.RIGHT, title1, 2);

	    return header;
	});

	// neuer Foot mit Datum und Seitenangeben
	page.setFooter((final PageNumber pageNumber) -> {
	    final GridPrint footer = new GridPrint("d:g, r:d"); //$NON-NLS-1$
	    final TextPrint datum = new TextPrint(this.getLastSaveDate(), SWT.LEFT);
	    datum.setFontData(AbstractPrint.FONT_TINY);
	    final PageNumberPrint seite = new PageNumberPrint(pageNumber, SWT.RIGHT);
	    seite.setFontData(AbstractPrint.FONT_TINY);
	    footer.add(datum);
	    footer.add(seite);

	    return footer;
	});
    }

    /**
     * @see de.frankmuenster.jameica.orverw.print.AbstractPrint#getPrintContent()
     */
    @Override
    @SuppressWarnings({ "PMD.AvoidInstantiatingObjectsInLoops" })
    protected Print getPrintContent() throws ApplicationException {

	final DefaultGridLook look = new DefaultGridLook();
	look.setCellSpacing(5, 5);
	final GridPrint grid = new GridPrint("50, r:d", look); //$NON-NLS-1$
	final double factor = (double) GUI.getDisplay().getDPI().x / (double) 72;
	final int indexFontHeight = (int) (this.indexFont.getFontData()[0].getHeight() * factor);
	final int regFontHeight = (int) (this.contentFont.getFontData()[0].getHeight() * factor);
	int height = Math.max(indexFontHeight, regFontHeight);
	height += 8;
	final int indexFontX = 3 + (height - indexFontHeight) / 2;
	final int regsiterFontX = 3 + (height - regFontHeight) / 2;
	try {
	    while (this.index.hasNext()) {
		final Register register = this.index.next();
		// Index
		LayerPrint layerPrint = new LayerPrint();
		ImageData imageData = this.getImage(height, 40, this.indexBgColor);
		ImagePrint imagePrint = new ImagePrint(imageData, new Point(72, 72));
		DefaultGridLook look1 = new DefaultGridLook();
		look1.setCellSpacing(0, 0);
		look1.setCellPadding(10, indexFontX);
		GridPrint text = new GridPrint("r:d", look1); //$NON-NLS-1$
		TextPrint textPrint = new TextPrint(register.getIndex(), this.indexFont.getFontData()[0]);
		text.add(textPrint);
		layerPrint.add(imagePrint);
		layerPrint.add(text);
		grid.add(layerPrint);
		// Description
		layerPrint = new LayerPrint();
		imageData = this.getImage(height, PrintRegister.DESCRIPTION_WIDTH, this.contentBgColor);
		imagePrint = new ImagePrint(imageData, new Point(72, 72));
		look1 = new DefaultGridLook();
		look1.setCellSpacing(0, 0);
		look1.setCellPadding(15, regsiterFontX);
		text = new GridPrint("r:d", look1); //$NON-NLS-1$
		final String description = this.fitDescription(register.getBeschreibung(), this.contentFont);
		textPrint = new TextPrint(description, this.contentFont.getFontData()[0]);
		text.add(textPrint);
		layerPrint.add(imagePrint);
		layerPrint.add(text);
		grid.add(layerPrint);
	    }
	} catch (final RemoteException e) {
	    throw new ApplicationException(I18N.tr("PrintRegister.errorCreatingPrintContent", this.title), //$NON-NLS-1$
		    e);
	}
	return grid;
    }

    /**
     * Checks and evtl shorten the description text to fit into given size
     *
     * @param beschreibung
     * @param font
     * @return evtl shortened description text
     */
    private String fitDescription(final String beschreibung, final Font font) {

	final StringBuilder shortend = new StringBuilder(beschreibung);
	final GC graphic = new GC(GUI.getShell());
	graphic.setFont(font);
	final int descWidth = graphic.textExtent(shortend.toString()).x;
	if (PrintRegister.DESCRIPTION_WIDTH < descWidth) {
	    shortend.delete(shortend.length() - 3, shortend.length()).append("..."); //$NON-NLS-1$
	    while (PrintRegister.DESCRIPTION_WIDTH < graphic.textExtent(shortend.toString()).x) {
		shortend.deleteCharAt(shortend.length() - 4);
	    }
	    // shortend = shortend.append("..."); //$NON-NLS-1$
	    Logger.debug("Short text  : " + shortend + "(" + graphic.textExtent(shortend.toString()) + ")"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	}
	graphic.dispose();
	return shortend.toString();
    }

    /**
     * Returns the background image for a row with the given height, width and
     * colour
     *
     * @param height
     * @param width
     * @param color
     * @return the row background image
     */
    private ImageData getImage(final int height, final int width, final Color color) {

	final Image image = new Image(GUI.getDisplay(), width, height);
	final GC graphic = new GC(image);
	graphic.setBackground(color);
	graphic.fillOval(0, 0, height / 2, height);
	graphic.fillRectangle(height / 4, 0, width, height);
	graphic.dispose();
	return image.getImageData();
    }

    private String getLastSaveDate() {

	final SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy", Locale.GERMAN); //$NON-NLS-1$
	return sdf.format(this.lastSaveDate);
    }
}
